function [M,X,Y,Z,N,h,B] = makeimage(f,fx,fy,r,s,A,L,R,intens)
%MAKEIMAGE generate a synthetic images matrix.
% input
%	f, fx, fy: function and partial derivatives for the observed object
%	r, s: images dimensions
%	A: horizontal size of each image
%	L: vector/matrix containing the lights position 
%	R: albedo matrix (default ones(r,s))
%	intens: light intensity (default ones(q,1))
% output
%	M: image of the object
%	X, Y, Z: object points coordinates
%	N: normal matrix
% if L is a matrix, an image for each column of L is produces and 
% the images are vectorized in the columns of M.
%
%   See also datasynth, testsynth, and testshell.

%   A. Concas, R. Dessì, C. Fenu, G. Rodriguez & M. Vanzi
%   Email: concas.anna@gmail.com, richidessi@gmail.com, kate.fenu@unica.it, 
%   rodriguez@unica.it, vanzi@diee.unica.it
%
%   Last revised June, 2021

attenuate = 1;	% attenuate lights according to distance; see below

p = r*s;
n = size(L,1);
q = size(L,2);

if nargin<9 || isempty(intens), intens = ones(q,1); end
if nargin<8 || isempty(R), R = makealbedo(1,r,s); end

if (n==3) || ( (n==4) && all(L(4,:)==0) )
	lightsatinf = 1;
elseif (n==4) && all(L(4,:)~=0)
	lightsatinf = 0;
	if any(L(4,:)~=1)	% normalize homogeneous coordinates
		L = L * diag(1./L(4,:));
	end
else
	error('error in matrix L')
end
L = L(1:3,:);

h = A/(r-1);
B = (s-1)*h;
x = linspace(-A/2,A/2,r);
y = linspace(-B/2,B/2,s);
[X,Y] = meshgrid(x,y);
if ~lightsatinf || nargout>3, Z = f(X,Y); end

P = fx(X,Y);
Q = fy(X,Y);
K = 1./sqrt(1+P.^2+Q.^2);
P = -P.*K;
Q = -Q.*K;
N = [P(:) Q(:) K(:)]';

M = zeros(p,q);
if lightsatinf
	for i = 1:q
		M(:,i) = R(:) .* (N' * (intens(i)*L(:,i)));
	end
else
	NS = sparse( [1:3*p]', kron([1:p]',[1;1;1]), N(:));
	W = [X(:) Y(:) Z(:)]';
	for i = 1:q
		V = ((intens(i)*L(:,i))*ones(1,p) - W);
		if attenuate
			nor = sqrt(sum(V.^2)).^3;	% distance attenuation
		else
			nor = sqrt(sum(V.^2));		% normalized vectors
		end
		V = V*spdiags(1./nor',0,p,p);
		M(:,i) = R(:) .* (NS'*V(:));
	end
end
if q==1,  M = reshape(M,r,s);  end
% normalize images
M = M/max(max(abs(M)));

