function [U,B] = poisneu(F,h,mu_1,mu_2,nu_1,nu_2,centervalue)
%POISNEU Poisson solver with Neumann boundary conditions.
%   U = poidir(F,h,...) solves the Poisson equation ΔU = F by solving a linear
%   system with coefficient matrix B obtained by a five point scheme
%   discretization with stepsize h, with Neumann boundary conditions (default
%   homogeneous BCs).
%    
%   See also ps3dstep2.

%   A. Concas, R. Dessì, C. Fenu, G. Rodriguez & M. Vanzi
%   Email: concas.anna@gmail.com, richidessi@gmail.com, kate.fenu@unica.it, 
%   rodriguez@unica.it, vanzi@diee.unica.it
%
%   Last revised June, 2021

if nargin<7, centervalue = 0; end
if nargin<6, mu_1 = 0; mu_2 = 0; nu_1 = 0; nu_2 = 0; end

[s,r] = size(F);
s = s-2;
r = r-2;

S = spdiags([0;ones(s,1);0],0,s+2,s+2);
p0 = [1;3*ones(s,1);1];
pm1 = [zeros(s,1);-1;nan];
p1 = [nan;-1;zeros(s,1)];
P = spdiags([pm1 p0 p1],-1:1,s+2,s+2);
q0 = -[1;4*ones(s,1);1];
Q = spdiags([-pm1 q0 -p1],-1:1,s+2,s+2);

R = spdiags([ [-pm1(2:end);nan] [ones(s,1);-4;nan] [3;-4*ones(s,1);3] ...
	[nan;-4;ones(s,1)] [nan;-p1(1:end-1)] ], -2:2,s+2,s+2);

B = kron(speye(r),R) + kron(spdiags(ones(r,1),1,r,r),S) ...
	+ kron(spdiags(ones(r,1),-1,r,r),S);
ZS = sparse(1,1,0,(r-1)*(s+2),s+2);
B = [ P Q S ZS'; [S;ZS] B [ZS;S]; ZS' S Q P];

F = F*h^2;
% boundary conditions
F(1,:) = 2*h*mu_1;
F(end,:) = 2*h*mu_2;
F(:,1) = 2*h*nu_1;
F(:,end) = 2*h*nu_2;
b = F(:);
% interpolation condition
n = size(B,1);
k = round(n/2);
B(k,:) = 0;
B(k,k) = 1;
b(k) = centervalue;
% system solution
u = B\b;
U = reshape(u,s+2,r+2);

