function [D,N,L,NT] = ps3dstep1( M, r, s, light2, show, fsiz)
%PS3DSTEP1 first step of the photometric stereo 3D reconstruction.
%   [D,N,L]=ps3dstep1(M,r,s) returns the albedo D, the normalized normal
%   vectors N, the recovered rotated light directions L.
%   [D,N,L]=ps3dstep1(M,r,s,...) optionally passes the light at west light2,
%   a flag to show intermediate results, which is also used for numbering the
%   figures (default 0), and the font size for figures.
%
%   See also ps3d.

%   A. Concas, R. Dessì, C. Fenu, G. Rodriguez & M. Vanzi
%   Email: concas.anna@gmail.com, richidessi@gmail.com, kate.fenu@unica.it, 
%   rodriguez@unica.it, vanzi@diee.unica.it
%
%   Last revised June, 2021

if nargin<6, fsiz = 10; end
if nargin<5, show = 0; end
if nargin<4, light2 = 0; end

[p,q] = size(M);
if p ~= r*s, error('Inconsistent image size.'), end

% construct initial rank-3 factorization by SVD
[U1,S1,V1] = svd(M,0);

if show
	figure(show)
	hs = semilogy(diag(S1),'o-');
	set(gca,'fontsize',fsiz)
	set(hs,'linewidth',2)
	title('Singular values of data matrix')
	xlabel(sprintf('\\sigma_3/\\sigma_4=%e',S1(3,3)/S1(4,4)))
	grid
	drawnow
	%pause
end

sigma = diag(S1(1:3,1:3));
N1 = diag(sigma) * U1(:,1:3)';
L1 = V1(:,1:3)';

% normalize the columns of L
[N2,L2,R,H] = normlights( N1, L1);
if show
	plotlights3d(L2,show+1,fsiz)
	title('recovered light directions before orientation')
	drawnow
end

% find rotation
[NT,L,Qrot] = rotatelights( N2, L2, light2);
if show
	plotlights3d(L,show+2,fsiz)
	title('recovered rotated light directions')
	drawnow
end

% normalize normals and extract albedo
norms = sqrt(sum(NT.^2)');
N = NT*spdiags(1./norms,0,p,p);
D = reshape(norms,s,r);
D = D/max(max(D));

