function [U,X,Y,P,Q] = ps3dstep2( NT, r, s, A, bcond, bdata)
%PS3DSTEP2 second step of the photometric stereo 3D reconstruction.
%   [U,X,Y] = ps3dstep2(NT,r,s,A,...) returns the 3D coordinates of the
%   observed objects, where the columns of NT contain the rXs normalized normal
%   vectors to the object, and A is the length of the horizontal axis. 
%   [U,X,Y,P,Q]=ps3dstep2(...) also returns the components P and Q of the
%   gradients.

%   See also ps3d.

%   A. Concas, R. Dessì, C. Fenu, G. Rodriguez & M. Vanzi
%   Email: concas.anna@gmail.com, richidessi@gmail.com, kate.fenu@unica.it, 
%   rodriguez@unica.it, vanzi@diee.unica.it
%
%   Last revised June, 2021

if nargin<6, bdata = {}; end
if nargin<5, bcond = 1; end

h = A/(r-1);		% pixel width in the same unit of A
B = (s-1)*h; 		% vertical size of image in meters (or cm, ...)
x = linspace(-A/2,A/2,r);
y = linspace(-B/2,B/2,s);
[X,Y] = meshgrid(x,y);

% extract gradients
P = reshape(-NT(1,:)./NT(3,:),s,r);
Q = reshape(-NT(2,:)./NT(3,:),s,r);

% why not using a second order approximation?
F = zeros(s,r);
F(2:s-1,2:r-1)  = (P(2:s-1,3:r)-P(2:s-1,1:r-2)+Q(3:s,2:r-1)-Q(1:s-2,2:r-1))/2/h;
if bcond == 1		% Dirichlet homogeneous boundary conditions
	U = poisdir(F,h);
elseif bcond == 2	% Neumann homogeneous boundary conditions
	if isempty(bdata)
		U = poisneu(F,h);
	else
		U = poisneu(F,h,bdata{1},bdata{2},bdata{3},bdata{4},bdata{5});
	end
else
	error('Unknown boundary conditions.')
end

